"use strict";
/* eslint-disable */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var tape_1 = __importDefault(require("tape"));
var fs_1 = __importDefault(require("fs"));
var crypto_1 = __importDefault(require("crypto"));
var zlib_1 = __importDefault(require("zlib"));
var supertest_1 = __importDefault(require("supertest"));
var koa_1 = __importDefault(require("koa"));
var http_1 = __importDefault(require("http"));
var path_1 = __importDefault(require("path"));
var _1 = __importDefault(require("."));
var dir = __dirname;
// file/magic doesn't actually know what a TS file is
var tsType = 'video/mp2t';
var textType = 'text/plain; charset=utf-8';
var correctStatus = function (s) { return "has ".concat(s, " status"); };
var is200 = correctStatus(200);
var is404 = correctStatus(404);
var is304 = correctStatus(304);
var correctHeader = function (s) { return "has correct ".concat(s); };
var correctCC = correctHeader('cache-control');
var correctCT = correctHeader('content-type');
var correctCL = correctHeader('content-length');
var correctLM = correctHeader('last-modified');
var correctET = correctHeader('etag');
var correctGZ = correctHeader('gzip');
var correctV = correctHeader('vary');
var indexTs = path_1.default.resolve(dir, 'index.ts');
(0, tape_1.default)('it should accept abnormal path', function (t) {
    var app = new koa_1.default();
    app.use((0, _1.default)({ dir: dir }));
    var server = http_1.default.createServer(app.callback());
    (0, supertest_1.default)(server)
        .get('//index.ts')
        .end(function (err, res) {
        if (err) {
            throw err;
        }
        t.deepEqual(res.status, 200, is200);
        t.end();
    });
});
var etag;
(0, tape_1.default)('it should serve files', function (t) {
    var app = new koa_1.default();
    app.use((0, _1.default)({ dir: dir }));
    var server = http_1.default.createServer(app.callback());
    (0, supertest_1.default)(server)
        .get('/index.ts')
        .end(function (err, res) {
        if (err) {
            throw err;
        }
        t.deepEqual(res.status, 200, is200);
        t.equal(res.header['cache-control'], 'public, max-age=0', correctCC);
        t.equal(res.header['content-type'], tsType, correctCT);
        t.ok(res.header['content-length'], correctCL);
        t.ok(res.header['last-modified'], correctLM);
        t.ok(res.header['etag'], correctET);
        etag = res.header.etag;
        t.end();
    });
});
(0, tape_1.default)('it should serve files as buffers', function (t) {
    var app = new koa_1.default();
    app.use((0, _1.default)({
        dir: dir,
        buffer: true,
    }));
    var server = http_1.default.createServer(app.callback());
    (0, supertest_1.default)(server)
        .get('/index.ts')
        .end(function (err, res) {
        if (err) {
            throw err;
        }
        t.deepEqual(res.status, 200, is200);
        t.equal(res.header['cache-control'], 'public, max-age=0', correctCC);
        t.equal(res.header['content-type'], tsType, correctCT);
        t.ok(res.header['content-length'], correctCL);
        t.ok(res.header['last-modified'], correctLM);
        t.ok(res.header['etag'], correctET);
        etag = res.header.etag;
        t.end();
    });
});
(0, tape_1.default)('it should serve recursive files', function (t) {
    var app = new koa_1.default();
    app.use((0, _1.default)({ dir: dir }));
    var server = http_1.default.createServer(app.callback());
    (0, supertest_1.default)(server)
        .get('/index.ts')
        .end(function (err, res) {
        if (err) {
            throw err;
        }
        t.deepEqual(res.status, 200, is200);
        t.equal(res.header['cache-control'], 'public, max-age=0');
        t.equal(res.header['content-type'], tsType, correctCT);
        t.ok(res.header['content-length'], correctCL);
        t.ok(res.header['last-modified'], correctLM);
        t.ok(res.header['etag'], correctET);
        t.end();
    });
});
(0, tape_1.default)('it should not serve hidden files', function (t) {
    var app = new koa_1.default();
    app.use((0, _1.default)({ dir: dir }));
    var server = http_1.default.createServer(app.callback());
    (0, supertest_1.default)(server)
        .get('/.gitignore')
        .end(function (err, res) {
        if (err) {
            throw err;
        }
        t.deepEqual(res.status, 404, is404);
        t.end();
    });
});
(0, tape_1.default)('it should support conditional HEAD requests', function (t) {
    var app = new koa_1.default();
    app.use((0, _1.default)({ dir: dir }));
    var server = http_1.default.createServer(app.callback());
    (0, supertest_1.default)(server)
        .head('/index.ts')
        .set('If-None-Match', etag)
        .end(function (err, res) {
        if (err) {
            throw err;
        }
        t.deepEqual(res.status, 304, is304);
        t.end();
    });
});
(0, tape_1.default)('it should support conditional GET requests', function (t) {
    var app = new koa_1.default();
    app.use((0, _1.default)({ dir: dir }));
    var server = http_1.default.createServer(app.callback());
    (0, supertest_1.default)(server)
        .get('/index.ts')
        .set('If-None-Match', etag)
        .end(function (err, res) {
        if (err) {
            throw err;
        }
        t.deepEqual(res.status, 304, is304);
        t.end();
    });
});
(0, tape_1.default)('it should support HEAD', function (t) {
    var app = new koa_1.default();
    app.use((0, _1.default)({ dir: dir }));
    var server = http_1.default.createServer(app.callback());
    (0, supertest_1.default)(server)
        .head('/index.ts')
        .end(function (err, res) {
        if (err) {
            throw err;
        }
        t.deepEqual(res.status, 200, is200);
        t.deepEqual(res.body, {}, 'has empty body');
        t.end();
    });
});
(0, tape_1.default)('it should support 404 Not Found for other Methods to allow downstream', function (t) {
    var app = new koa_1.default();
    app.use((0, _1.default)({ dir: dir }));
    var server = http_1.default.createServer(app.callback());
    (0, supertest_1.default)(server)
        .put('/index.ts')
        .end(function (err, res) {
        if (err) {
            throw err;
        }
        t.deepEqual(res.status, 404, is404);
        t.end();
    });
});
(0, tape_1.default)('it should ignore query strings', function (t) {
    var app = new koa_1.default();
    app.use((0, _1.default)({ dir: dir }));
    var server = http_1.default.createServer(app.callback());
    (0, supertest_1.default)(server)
        .get('/index.ts?query=string')
        .end(function (err, res) {
        if (err) {
            throw err;
        }
        t.deepEqual(res.status, 200, is200);
        t.end();
    });
});
(0, tape_1.default)('it should set the etag and content-md5 headers', function (t) {
    var app = new koa_1.default();
    app.use((0, _1.default)({ dir: dir }));
    var server = http_1.default.createServer(app.callback());
    var index = fs_1.default.readFileSync(indexTs);
    var md5 = crypto_1.default.createHash('md5').update(index).digest('base64');
    (0, supertest_1.default)(server)
        .get('/index.ts')
        .end(function (err, res) {
        if (err) {
            throw err;
        }
        t.deepEqual(res.status, 200, is200);
        t.equal(res.header['etag'], "\"".concat(md5, "\""), correctET);
        t.equal(res.header['content-md5'], md5, correctHeader('content-md5'));
        t.end();
    });
});
(0, tape_1.default)('it should serve files with gzip buffer', function (t) {
    var filePath = path_1.default.resolve(dir, 'foo.txt');
    fs_1.default.writeFileSync(filePath, 'hello world'.repeat(1000));
    var app = new koa_1.default();
    app.use((0, _1.default)({
        dir: dir,
        buffer: true,
        gzip: true,
    }));
    var server = http_1.default.createServer(app.callback());
    var f = fs_1.default.readFileSync(filePath);
    zlib_1.default.gzip(f, function (err, content) {
        if (err) {
            throw err;
        }
        (0, supertest_1.default)(server)
            .get('/foo.txt')
            .set('Accept-Encoding', 'gzip')
            .end(function (err, res) {
            if (err) {
                throw err;
            }
            t.deepEqual(res.status, 200, is200);
            t.ok(f.toString(), 'file is ok');
            t.equal(res.header['vary'], 'Accept-Encoding', correctV);
            t.equal(res.header['content-length'], "".concat(content.length), correctCL);
            t.equal(res.header['content-encoding'], 'gzip', correctGZ);
            t.equal(res.header['cache-control'], 'public, max-age=0', correctCC);
            t.equal(res.header['content-type'], textType, correctCT);
            t.ok(res.header['content-length'], correctCL);
            t.ok(res.header['last-modified'], correctLM);
            t.ok(res.header['etag'], correctET);
            etag = res.header.etag;
            fs_1.default.unlinkSync(filePath);
            t.end();
        });
    });
});
(0, tape_1.default)('it should not serve files with gzip buffer when accept encoding not include gzip', function (t) {
    var app = new koa_1.default();
    app.use((0, _1.default)({
        dir: dir,
        buffer: true,
        gzip: true,
    }));
    var server = http_1.default.createServer(app.callback());
    var index = fs_1.default.readFileSync(indexTs);
    (0, supertest_1.default)(server)
        .get('/index.ts')
        .set('Accept-Encoding', '')
        .end(function (err, res) {
        if (err) {
            throw err;
        }
        t.equal(res.header['content-type'], tsType);
        t.deepEqual(res.status, 200, is200);
        t.ok(index.toString(), 'has index');
        t.equal(res.header['cache-control'], 'public, max-age=0', correctCC);
        t.equal(res.header['content-length'], "".concat(index.length), correctCL);
        t.notOk(res.header['content-encoding'], correctHeader('content-encoding'));
        t.ok(res.header['content-length'], correctCL);
        t.ok(res.header['last-modified'], correctLM);
        t.ok(res.header['etag'], correctET);
        t.end();
    });
});
(0, tape_1.default)('it should serve files with gzip stream', function (t) {
    var filePath = path_1.default.resolve(dir, 'foo.txt');
    fs_1.default.writeFileSync(filePath, 'hello world'.repeat(1000));
    var app = new koa_1.default();
    app.use((0, _1.default)({
        dir: dir,
        gzip: true,
    }));
    var server = http_1.default.createServer(app.callback());
    var f = fs_1.default.readFileSync(filePath);
    zlib_1.default.gzip(f, function (err) {
        if (err) {
            throw err;
        }
        (0, supertest_1.default)(server)
            .get('/foo.txt')
            .set('Accept-Encoding', 'gzip')
            .end(function (err, res) {
            if (err) {
                throw err;
            }
            t.equal(res.header['content-type'], textType, correctCT);
            t.equal(res.header['content-encoding'], 'gzip', correctGZ);
            t.deepEqual(res.status, 200, is200);
            t.ok(f.toString(), 'file is ok');
            t.equal(res.header['cache-control'], 'public, max-age=0', correctCC);
            t.ok(res.header['last-modified'], correctLM);
            t.ok(res.header['etag'], correctET);
            t.equal(res.header['vary'], 'Accept-Encoding', correctV);
            etag = res.header.etag;
            fs_1.default.unlinkSync(filePath);
            t.end();
        });
    });
});
(0, tape_1.default)('it should work fine when new file added', function (t) {
    var filePath = path_1.default.resolve(dir, 'a.js');
    fs_1.default.writeFileSync(filePath, 'hello world');
    var app = new koa_1.default();
    app.use((0, _1.default)({ dir: dir }));
    var server = http_1.default.createServer(app.callback());
    (0, supertest_1.default)(server)
        .get('/a.js')
        .end(function (err, res) {
        if (err) {
            throw err;
        }
        t.deepEqual(res.status, 200, is200);
        fs_1.default.unlinkSync(filePath);
        t.end();
    });
});
(0, tape_1.default)('it should 404 when new hidden file added', function (t) {
    var filePath = path_1.default.resolve(dir, '.a.js');
    fs_1.default.writeFileSync(filePath, 'hello world');
    var app = new koa_1.default();
    app.use((0, _1.default)({ dir: dir }));
    var server = http_1.default.createServer(app.callback());
    (0, supertest_1.default)(server)
        .get('/.a.js')
        .end(function (err, res) {
        if (err) {
            throw err;
        }
        t.deepEqual(res.status, 404, is404);
        fs_1.default.unlinkSync(filePath);
        t.end();
    });
});
(0, tape_1.default)('it should 404 when file not exist', function (t) {
    var app = new koa_1.default();
    app.use((0, _1.default)({ dir: dir }));
    var server = http_1.default.createServer(app.callback());
    (0, supertest_1.default)(server)
        .get('/a.js')
        .end(function (err, res) {
        if (err) {
            throw err;
        }
        t.deepEqual(res.status, 404, is404);
        t.end();
    });
});
(0, tape_1.default)('it should 404 when is directory without index.html', function (t) {
    var app = new koa_1.default();
    app.use((0, _1.default)({ dir: dir }));
    var server = http_1.default.createServer(app.callback());
    (0, supertest_1.default)(server)
        .get('/')
        .end(function (err, res) {
        if (err) {
            throw err;
        }
        t.deepEqual(res.status, 404, is404);
        t.end();
    });
});
(0, tape_1.default)('it should fall back to index.html if available', function (t) {
    var indexPath = path_1.default.resolve(dir, 'index.html');
    fs_1.default.writeFileSync(indexPath, 'hello world');
    var app = new koa_1.default();
    app.use((0, _1.default)({ dir: dir }));
    var server = http_1.default.createServer(app.callback());
    (0, supertest_1.default)(server)
        .get('/')
        .end(function (err, res) {
        if (err) {
            throw err;
        }
        t.deepEqual(res.status, 200, is200);
        t.deepEqual(res.text, 'hello world', 'has correct text');
        fs_1.default.unlinkSync(indexPath);
        t.end();
    });
});
(0, tape_1.default)('it should not load files above options.dir', function (t) {
    var app = new koa_1.default();
    app.use((0, _1.default)({ dir: dir }));
    var server = http_1.default.createServer(app.callback());
    (0, supertest_1.default)(server)
        .get('/%2E%2E/package.json')
        .end(function (err, res) {
        if (err) {
            throw err;
        }
        t.deepEqual(res.status, 404, is404);
        t.end();
    });
});
